{% extends "ievv_developemail/admin/base.django.html" %}


{% block content %}
    {% with plaintext_message=developemail.plaintext_message %}
        {% if plaintext_message %}
            <div class="developemail-code">{{ plaintext_message }}</div>
        {% else %}
            <p class="developemail-warning">
                No plaintext message
            </p>
        {% endif %}
    {% endwith %}
{% endblock content %}
