# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-04-30 13:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DevelopEmail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True)),
                ('subject', models.TextField(blank=True, default='')),
                ('from_email', models.TextField(blank=True, db_index=True, default='')),
                ('to_emails', models.TextField(blank=True, default='')),
                ('raw_message', models.TextField(blank=True, default='')),
            ],
            options={
                'ordering': ['-created_datetime'],
            },
        ),
    ]
