{% extends "ievv_developemail/admin/base.django.html" %}


{% block content %}
    {% with html_message=developemail.html_message %}
        {% if html_message %}
            {{ html_message|safe }}
        {% else %}
            <p class="developemail-warning">
                No html message
            </p>
        {% endif %}
    {% endwith %}
{% endblock content %}
