# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2019-11-24 19:53
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ievv_logging', '0003_auto_20191123_1147'),
    ]

    operations = [
        migrations.RenameField(
            model_name='ievvloggingeventbase',
            old_name='error_occured',
            new_name='success_run',
        ),
        migrations.RenameField(
            model_name='ievvloggingeventitem',
            old_name='error_occured',
            new_name='success_run',
        ),
    ]
