# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2019-03-01 09:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ievv_sms', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='debugsmsmessage',
            name='send_as',
            field=models.CharField(blank=True, default='', max_length=50),
        ),
    ]
