{% spaceless %}
{% endspaceless %}<!DOCTYPE html>
<html>
    <head>
        <title>
            {% block title %}
                #{{ developemail.id }} - {{ developemail.subject }}
            {% endblock title %}
        </title>
        <style type="text/css">
            body, html {
                margin: 0;
                padding: 0;
                background-color: #fff;
            }
            .developemail-header {
                background-color: #fff;
                color: #222;
                font-size: 18px;
                font-family: Roboto, "Helvetica Neue", sans-serif;
                border-bottom: 3px solid #222;
                padding: 15px;
            }

            .developemail-navigation, .developemail-metadata {
                max-width: 700px;
                margin-left: auto;
                margin-right: auto;
            }

            .developemail-navigation {
                display: flex;
                justify-content: center;
                flex-wrap: wrap;
            }

            .developemail-navigation__item {
                flex: 0 1 auto;
                margin-left: 6px;
                margin-right: 6px;
                margin-bottom: 6px;
                display: block;
                background-color: #efefef;
                padding: 5px 10px;
                border-radius: 5px;
                border: 3px solid #999;
                color: #777;
                text-decoration: none;
                text-transform: uppercase;
                font-weight: bold;
                letter-spacing: 0.06em;
                font-size: 16px;
            }
            .developemail-navigation__item:hover, .developemail-navigation__item--active {
                border: 3px solid #222;
                color: #000;
                text-decoration: none;
            }

            .developemail-metadata {
                margin-bottom: 0;
                padding: 0;
            }
            .developemail-metadata__title {
                margin: 0;
                padding: 0;
                font-size: 14px;
                font-weight: bold;
                color: #777;
                text-transform: uppercase;
            }
            .developemail-metadata__text {
                margin: 0 0 10px 0;
                padding: 0;
            }
            .developemail-metadata__text:last-child {
                margin-bottom: 0;
            }

            .developemail-warning {
                margin: 20px auto;
                background-color: #b00;
                color: #fff;
                max-width: 700px;
                text-align: center;
                font-size: 18px;
                font-family: Roboto, "Helvetica Neue", sans-serif;
                padding: 20px;
                font-weight: bold;
            }

            .developemail-code {
                font-family: "Lucida Console", "Monaco", monospace;
                font-size: 16px;
                white-space: pre-wrap;
                max-width: 100%;
                padding: 15px;
                overflow-x: auto;
            }

        </style>
    </head>
    <body>
        <div class="developemail-header">
            <nav class="developemail-navigation">
                <a class="developemail-navigation__item" href="{% url 'admin:ievv_developemail_developemail_changelist' %}">
                    Inbox ({{ developemail_count }})
                </a>
                <a class="developemail-navigation__item {% if mode == 'plain' %}developemail-navigation__item--active{% endif %}"
                        href="{% url 'admin:developemail_as_plain' developemail.id %}">
                    Plain
                </a>
                <a class="developemail-navigation__item {% if mode == 'html' %}developemail-navigation__item--active{% endif %}"
                        href="{% url 'admin:developemail_as_html' developemail.id  %}">
                    HTML
                </a>
                <a class="developemail-navigation__item {% if mode == 'raw' %}developemail-navigation__item--active{% endif %}"
                        href="{% url 'admin:developemail_as_raw' developemail.id %}">
                    Raw
                </a>
                <a class="developemail-navigation__item" href="{% url 'admin:ievv_developemail_developemail_change' developemail.id %}">
                    Edit
                </a>
            </nav>
            <dl class="developemail-metadata">
                <dt class="developemail-metadata__title">
                    Subject:
                </dt>
                <dd class="developemail-metadata__text">
                    {{ developemail.subject_with_fallback }}
                </dd>
                <dt class="developemail-metadata__title">
                    From:
                </dt>
                <dd class="developemail-metadata__text">
                    {{ developemail.from_email }}
                </dd>
                <dt class="developemail-metadata__title">
                    To:
                </dt>
                <dd class="developemail-metadata__text">
                    {{ developemail.to_emails }}
                </dd>
            </dl>
        </div>
        {% block content %}{% endblock content %}
    </body>
</html>
